//=============================================================================
// CustomizeConfigItem.js
// ----------------------------------------------------------------------------
// Copyright (c) 2015 Triacontane
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// Version
// 2.0.1 2017/10/15 2.0.0の修正によりスイッチ項目を有効にしたときにゲーム開始するとエラーになる問題を修正
// 2.0.0 2017/09/10 ツクールの型指定機能に対応し、各オプション項目を任意の数だけ追加できる機能を追加
// 1.2.3 2017/06/08 1.2.2の修正により起動できなくなっていた問題を修正
// 1.2.2 2017/05/27 競合の可能性のある記述（Objectクラスへのプロパティ追加）をリファクタリング
// 1.2.1 2016/12/08 1.2.0の機能追加以降、デフォルト項目で決定ボタンを押すとエラーになっていた現象を修正
// 1.2.0 2016/12/02 各項目で決定ボタンを押したときに実行されるスクリプトを設定できる機能を追加
// 1.1.1 2016/08/14 スイッチ項目、音量項目の初期値が無効になっていた問題を修正
// 1.1.0 2016/04/29 項目をクリックしたときに項目値が循環するよう修正
// 1.0.0 2016/01/17 初版
// ----------------------------------------------------------------------------
// [Blog]   : http://triacontane.blogspot.jp/
// [Twitter]: https://twitter.com/triacontane/
// [GitHub] : https://github.com/triacontane/
//=============================================================================

/*:
 * @plugindesc オプション任意項目作成プラグイン
 * @author トリアコンタン
 *
 * @param 数値項目
 * @desc 追加する数値項目のオプション項目情報です。
 * @default
 * @type struct<NumberData>[]
 *
 * @param 文字項目
 * @desc 追加する文字項目のオプション項目情報です。
 * @default
 * @type struct<StringData>[]
 *
 * @param スイッチ項目
 * @desc 追加するスイッチ項目のオプション項目情報です。
 * @default
 * @type struct<BooleanData>[]
 *
 * @param 音量項目
 * @desc 追加する音量項目のオプション項目情報です。
 * @default
 * @type struct<VolumeData>[]
 *
 * @param 数値項目2
 * @desc 追加する数値項目のオプション項目情報です。
 * @default
 * @type struct<NumberData>[]
 *
 * @param 文字項目2
 * @desc 追加する文字項目のオプション項目情報です。
 * @default
 * @type struct<StringData>[]
 *
 * @param スイッチ項目2
 * @desc 追加するスイッチ項目のオプション項目情報です。
 * @default
 * @type struct<BooleanData>[]
 *
 * @param 数値項目3
 * @desc 追加する数値項目のオプション項目情報です。
 * @default
 * @type struct<NumberData>[]
 *
 * @param 文字項目3
 * @desc 追加する文字項目のオプション項目情報です。
 * @default
 * @type struct<StringData>[]
 *
 * @param スイッチ項目3
 * @desc 追加するスイッチ項目のオプション項目情報です。
 * @default
 * @type struct<BooleanData>[]
 *
 * @help オプション画面に任意の項目を追加します。
 * 項目の種類は、以下の四種類があります。
 * 不要な項目は値を空に設定してください。
 *
 * ・スイッチ項目：
 * ON/OFFを選択する項目です。指定した番号のスイッチと値が同期されます。
 * オプションから値を設定すれば、それがスイッチに反映され、
 * スイッチを変更すれば、オプションの値に反映されます。
 * さらに、値はセーブデータ間で共有されます。
 * 隠しフラグを設定すると、オプション画面に表示されなくなります。
 * ゲームを進めないと出現しない項目などに利用できます。
 * 隠しフラグはプラグインコマンドから解除できます。
 *
 * スクリプトは上級者向け項目です。対象にカーソルを合わせて決定ボタンを
 * 押下すると指定したJavaScriptを実行できます。
 * 主に専用の設定画面などの遷移に使用します。
 *
 * ・数値項目：
 * 数値を選択する項目です。指定した番号の変数と値が同期されます。
 * スイッチ項目で指定した内容に加えて、
 * 最小値と最大値および一回の入力で変化する値を指定します。
 *
 * ・音量項目：
 * 音量を選択する項目です。BGMボリュームなどと同じ仕様で
 * キャラクターごとのボイス音量等に使ってください。
 *
 * ・文字項目：
 * 文字を選択する項目です。指定した文字の配列から項目を選択します。
 * 選択した文字のインデックス(開始位置は0)が変数に設定されます。
 * 初期値に設定する値もインデックスです。
 *
 * プラグインコマンド詳細
 *  イベントコマンド「プラグインコマンド」から実行。
 *  （パラメータの間は半角スペースで区切る）
 *
 *  CC_UNLOCK or
 *  オプション任意項目の隠し解除 [項目名]
 *  　指定した項目の隠しフラグを解除します。
 *  使用例：CC_UNLOCK 数値項目1
 *
 * 利用規約：
 *  作者に無断で改変、再配布が可能で、利用形態（商用、18禁利用等）
 *  についても制限はありません。
 *  このプラグインはもうあなたのものです。
 */

/*~struct~NumberData:
 * @param Name
 * @desc 項目の名称です。
 * @default 数値項目1
 *
 * @param DefaultValue
 * @desc 項目の初期値です。
 * @default 0
 * @type number
 *
 * @param VariableID
 * @desc 項目の設定内容が格納される変数番号です。
 * @default 0
 * @type variable
 *
 * @param HiddenFlag
 * @desc 項目がデフォルトで隠されるようになります。プラグインコマンドの実行で表示されます。
 * @default false
 * @type boolean
 *
 * @param Script
 * @desc 項目を決定したときに実行されるスクリプトです。
 * @default
 *
 * @param NumberMin
 * @desc 項目の最小値です。
 * @default 0
 * @type number
 *
 * @param NumberMax
 * @desc 項目の最大値です。
 * @default 100
 * @type number
 *
 * @param NumberStep
 * @desc 項目を操作したときの数値の変化量です。
 * @default 20
 * @type number
 */
/*~struct~BooleanData:
 * @param Name
 * @desc 項目の名称です。
 * @default スイッチ項目1
 *
 * @param DefaultValue
 * @desc 項目の初期値です。
 * @default false
 * @type boolean
 *
 * @param SwitchID
 * @desc 項目の設定内容が格納されるスイッチ番号です。
 * @default 0
 * @type switch
 *
 * @param HiddenFlag
 * @desc 項目がデフォルトで隠されるようになります。プラグインコマンドの実行で表示されます。
 * @default false
 * @type boolean
 *
 * @param Script
 * @desc 項目を決定したときに実行されるスクリプトです。
 * @default
 */
/*~struct~StringData:
 * @param Name
 * @desc 項目の名称です。
 * @default 文字列項目1
 *
 * @param DefaultValue
 * @desc 項目の初期値です。インデックスの数値を指定します。
 * @default 0
 * @type number
 *
 * @param VariableID
 * @desc 項目の設定内容が格納される変数番号です。
 * @default 0
 * @type variable
 *
 * @param HiddenFlag
 * @desc 項目がデフォルトで隠されるようになります。プラグインコマンドの実行で表示されます。
 * @default false
 * @type boolean
 *
 * @param Script
 * @desc 項目を決定したときに実行されるスクリプトです。
 * @default
 *
 * @param StringItems
 * @desc 項目の設定内容の配列です。
 * @default
 * @type string[]
 */
/*~struct~VolumeData:
 * @param Name
 * @desc 項目の名称です。
 * @default 音量項目1
 *
 * @param DefaultValue
 * @desc 項目の初期値です。
 * @default 0
 * @type number
 *
 * @param VariableID
 * @desc 項目の設定内容が格納される変数番号です。
 * @default 0
 * @type variable
 *
 * @param HiddenFlag
 * @desc 項目がデフォルトで隠されるようになります。プラグインコマンドの実行で表示されます。
 * @default false
 * @type boolean
 *
 * @param Script
 * @desc 項目を決定したときに実行されるスクリプトです。
 * @default
 */
 
 //=============================================================================
 // Window_Options2
 //  white_esorat独自追記 2つ目のオプションウィンドウを定義します
 //=============================================================================
 function Window_Options2() {
     this.initialize.apply(this, arguments);
 }
 //=============================================================================
 // Window_Options3
 //  white_esorat独自追記 3つ目のオプションウィンドウを定義します
 //=============================================================================
 function Window_Options3() {
     this.initialize.apply(this, arguments);
 }
 //=============================================================================
 // Window_Options3
 //  white_esorat独自追記 3つ目のオプションウィンドウを定義します
 //=============================================================================
 function Window_Options4() {
     this.initialize.apply(this, arguments);
 }
 
(function() {
    'use strict';
    var pluginName = 'CustomizeConfigItem';

    var getParamString = function(paramNames) {
        var value = getParamOther(paramNames);
        return value == null ? '' : value;
    };

    var getParamOther = function(paramNames) {
        if (!Array.isArray(paramNames)) paramNames = [paramNames];
        for (var i = 0; i < paramNames.length; i++) {
            var name = PluginManager.parameters(pluginName)[paramNames[i]];
            if (name) return name;
        }
        return null;
    };

    var getParamArrayJson = function(paramNames, defaultValue) {
        var value = getParamString(paramNames) || null;
        try {
            value = JSON.parse(value);
            if (value === null) {
                value = defaultValue;
            } else {
                value = value.map(function(valueData) {
                    return JSON.parse(valueData);
                });
            }
        } catch (e) {
            alert(`!!!Plugin param is wrong.!!!\nPlugin:${pluginName}.js\nName:[${paramNames}]\nValue:${value}`);
            value = defaultValue;
        }
        return value;
    };

    var getCommandName = function(command) {
        return (command || '').toUpperCase();
    };

    var getArgNumber = function(arg, min, max) {
        if (arguments.length < 2) min = -Infinity;
        if (arguments.length < 3) max = Infinity;
        return (parseInt(arg) || 0).clamp(min, max);
    };

    var getArgBoolean = function(arg) {
        return (arg || '').toUpperCase() === 'TRUE';
    };

    var getArgJson = function(arg, defaultValue) {
        try {
            arg = JSON.parse(arg);
            if (arg === null) {
                arg = defaultValue;
            }
        } catch (e) {
            alert(`!!!Plugin param is wrong.!!!\nPlugin:${pluginName}.js\nValue:${arg}`);
            arg = defaultValue;
        }
        return arg;
    };

    var iterate = function(that, handler) {
        Object.keys(that).forEach(function(key, index) {
            handler.call(that, key, that[key], index);
        });
    };

    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================
    var param           = {};
    param.numberOptions = getParamArrayJson(['NumberOptions', '数値項目'], []);
    param.stringOptions = getParamArrayJson(['StringOptions', '文字項目'], []);
    param.switchOptions = getParamArrayJson(['SwitchOptions', 'スイッチ項目'], []);
    param.volumeOptions = getParamArrayJson(['VolumeOptions', '音量項目'], []);
    param.numberOptions2 = getParamArrayJson(['NumberOptions2', '数値項目2'], []);
    param.stringOptions2 = getParamArrayJson(['StringOptions2', '文字項目2'], []);
    param.switchOptions2 = getParamArrayJson(['SwitchOptions2', 'スイッチ項目2'], []);
    param.numberOptions3 = getParamArrayJson(['NumberOptions3', '数値項目3'], []);
    param.stringOptions3 = getParamArrayJson(['StringOptions3', '文字項目3'], []);
    param.switchOptions3 = getParamArrayJson(['SwitchOptions3', 'スイッチ項目3'], []);

    var localOptionWindowIndex = 0;
    var localOptionWindowIndex2 = 0;

    //=============================================================================
    // Game_Interpreter
    //  プラグインコマンドを追加定義します。
    //=============================================================================
    var _Game_Interpreter_pluginCommand      = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        _Game_Interpreter_pluginCommand.call(this, command, args);
        this.pluginCommandCustomizeConfigItem(command, args);
    };

    Game_Interpreter.prototype.pluginCommandCustomizeConfigItem = function(command, args) {
        switch (getCommandName(command)) {
            case 'CC_UNLOCK' :
            case 'オプション任意項目の隠し解除' :
                ConfigManager.customParamUnlock(args[0]);
                break;
        }
    };

    //=============================================================================
    // ConfigManager
    //  追加項目の設定値や初期値を管理します。
    //=============================================================================
    ConfigManager.customParams   = null;
    ConfigManager.hiddenInfo     = {};
    ConfigManager._symbolNumber  = 'Number';
    ConfigManager._symbolBoolean = 'Boolean';
    ConfigManager._symbolString  = 'String';
    ConfigManager._symbolVolume  = 'Volume';

    ConfigManager.getCustomParams = function() {
        if (this.customParams) {
            return this.customParams;
        }
        this.customParams = {};
        param.numberOptions.forEach(function(optionItem, index) {
            this.makeNumberOption(optionItem, index);
        }, this);
        param.stringOptions.forEach(function(optionItem, index) {
            this.makeStringOption(optionItem, index);
        }, this);
        param.switchOptions.forEach(function(optionItem, index) {
            this.makeSwitchOption(optionItem, index);
        }, this);
        param.volumeOptions.forEach(function(optionItem, index) {
            this.makeVolumeOption(optionItem, index);
        }, this);
        param.numberOptions2.forEach(function(optionItem, index) {
            this.makeNumberOption(optionItem, index+100);
        }, this);
        param.stringOptions2.forEach(function(optionItem, index) {
            this.makeStringOption(optionItem, index+100);
        }, this);
        param.switchOptions2.forEach(function(optionItem, index) {
            this.makeSwitchOption(optionItem, index+100);
        }, this);
        param.numberOptions3.forEach(function(optionItem, index) {
            this.makeNumberOption(optionItem, index+200);
        }, this);
        param.stringOptions3.forEach(function(optionItem, index) {
            this.makeStringOption(optionItem, index+200);
        }, this);
        param.switchOptions3.forEach(function(optionItem, index) {
            this.makeSwitchOption(optionItem, index+200);
        }, this);
        return this.customParams;
    };
    
    ConfigManager.getCustomParams1 = function() {
        if (this.customParams1) {
            return this.customParams1;
        }
        this.customParams1 = {};
        var keylist = Object.keys(this.customParams).filter(function(symbol) {
            if(symbol.match(/Number\d$/)) return true;
            if(symbol.match(/String\d$/)) return true;
            if(symbol.match(/Boolean\d$/)) return true;
            return false;
        }, this);
        keylist.forEach(function(key) {
            this.customParams1[key] = this.customParams[key];
        }, this);
        
        return this.customParams1;
    };
    
    ConfigManager.getCustomParams2 = function() {
        if (this.customParams2) {
            return this.customParams2;
        }
        this.customParams2 = {};
        var keylist = Object.keys(this.customParams).filter(function(symbol) {
            if(symbol.match(/Number1\d+/)) return true;
            if(symbol.match(/String1\d+/)) return true;
            if(symbol.match(/Boolean1\d+/)) return true;
            return false;
        }, this);
        keylist.forEach(function(key) {
            this.customParams2[key] = this.customParams[key];
        }, this);
        
        return this.customParams2;
    };
    
    ConfigManager.getCustomParams3 = function() {
        if (this.customParams3) {
            
            return this.customParams3;
        }
        this.customParams3 = {};
        var keylist = Object.keys(this.customParams).filter(function(symbol) {
            if(symbol.match(/Number2\d+/)) return true;
            if(symbol.match(/String2\d+/)) return true;
            if(symbol.match(/Boolean2\d+/)) return true;
            return false;
        }, this);
        keylist.forEach(function(key) {
            this.customParams3[key] = this.customParams[key];
        }, this);
        return this.customParams3;
    };

    ConfigManager.makeNumberOption = function(optionItem, index) {
        var data    = this.makeCommonOption(optionItem, index, this._symbolNumber);
        data.min    = getArgNumber(optionItem.NumberMin);
        data.max    = getArgNumber(optionItem.NumberMax);
        data.offset = getArgNumber(optionItem.NumberStep);
        this.pushOptionData(data);
    };

    ConfigManager.makeStringOption = function(optionItem, index) {
        var data    = this.makeCommonOption(optionItem, index, this._symbolString);
        data.values = getArgJson(optionItem.StringItems);
        data.min    = 0;
        data.max    = data.values.length - 1;
        this.pushOptionData(data);
    };

    ConfigManager.makeSwitchOption = function(optionItem, index) {
        var data       = this.makeCommonOption(optionItem, index, this._symbolBoolean);
        data.initValue = getArgBoolean(optionItem.DefaultValue);
        data.variable  = getArgNumber(optionItem.SwitchID);
        this.pushOptionData(data);
    };

    ConfigManager.makeVolumeOption = function(optionItem, index) {
        var data = this.makeCommonOption(optionItem, index, this._symbolVolume);
        this.pushOptionData(data);
    };

    ConfigManager.makeCommonOption = function(optionItem, index, type) {
        var data       = {};
        data.symbol    = `${type}${index + 1}`;
        data.name      = optionItem.Name;
        data.hidden    = getArgBoolean(optionItem.HiddenFlag);
        data.script    = optionItem.Script;
        data.initValue = getArgNumber(optionItem.DefaultValue);
        data.variable  = getArgNumber(optionItem.VariableID, 0);
        return data;
    };

    ConfigManager.pushOptionData = function(data) {
        this.customParams[data.symbol] = data;
    };

    var _ConfigManager_makeData = ConfigManager.makeData;
    ConfigManager.makeData      = function() {
        var config        = _ConfigManager_makeData.apply(this, arguments);
        config.hiddenInfo = {};
        iterate(this.getCustomParams(), function(symbol) {
            config[symbol]            = this[symbol];
            config.hiddenInfo[symbol] = this.hiddenInfo[symbol];
        }.bind(this));
        return config;
    };

    var _ConfigManager_applyData = ConfigManager.applyData;
    ConfigManager.applyData      = function(config) {
        _ConfigManager_applyData.apply(this, arguments);
        iterate(this.getCustomParams(), function(symbol, item) {
            if (symbol.contains(this._symbolBoolean)) {
                this[symbol] = this.readFlagCustom(config, symbol, item);
            } else if (symbol.contains(this._symbolVolume)) {
                this[symbol] = this.readVolumeCustom(config, symbol, item);
            } else {
                this[symbol] = this.readOther(config, symbol, item);
            }
            this.hiddenInfo[symbol] = (config.hiddenInfo ? config.hiddenInfo[symbol] : item.hidden);
        }.bind(this));
    };

    ConfigManager.customParamUnlock = function(name) {
        iterate(this.getCustomParams(), function(symbol, item) {
            if (item.name === name) this.hiddenInfo[symbol] = false;
        }.bind(this));
        this.save();
    };

    ConfigManager.readOther = function(config, name, item) {
        var value = config[name];
        if (value !== undefined) {
            return Number(value).clamp(item.min, item.max);
        } else {
            return item.initValue;
        }
    };

    ConfigManager.readFlagCustom = function(config, name, item) {
        if (config[name] !== undefined) {
            return this.readFlag(config, name);
        } else {
            return item.initValue;
        }
    };

    ConfigManager.readVolumeCustom = function(config, name, item) {
        if (config[name] !== undefined) {
            return this.readVolume(config, name);
        } else {
            return item.initValue;
        }
    };

    ConfigManager.exportCustomParams = function() {
        if (!$gameVariables || !$gameSwitches) return;
        iterate(this.getCustomParams(), function(symbol, item) {
            if (item.variable > 0) {
                if (symbol.contains(this._symbolBoolean)) {
                    $gameSwitches.setValue(item.variable, !!this[symbol]);
                } else {
                    $gameVariables.setValue(item.variable, this[symbol]);
                }
            }
        }.bind(this));
    };

    ConfigManager.importCustomParams = function() {
        if (!$gameVariables || !$gameSwitches) return;
        iterate(this.getCustomParams(), function(symbol, item) {
            if (item.variable > 0) {
                if (symbol.contains(this._symbolBoolean)) {
                    this[symbol] = $gameSwitches.value(item.variable);
                } else if (symbol.contains(this._symbolVolume)) {
                    this[symbol] = $gameVariables.value(item.variable).clamp(0, 100);
                } else {
                    this[symbol] = $gameVariables.value(item.variable).clamp(item.min, item.max);
                }
            }
        }.bind(this));
    };

    var _ConfigManager_save = ConfigManager.save;
    ConfigManager.save      = function() {
        _ConfigManager_save.apply(this, arguments);
        this.exportCustomParams();
    };

    //=============================================================================
    // Game_Map
    //  リフレッシュ時にオプション値を同期します。
    //=============================================================================
    var _Game_Map_refresh      = Game_Map.prototype.refresh;
    Game_Map.prototype.refresh = function() {
        _Game_Map_refresh.apply(this, arguments);
        ConfigManager.importCustomParams();
    };

    //=============================================================================
    // DataManager
    //  セーブ時とロード時にオプション値を同期します。
    //=============================================================================
    var _DataManager_setupNewGame = DataManager.setupNewGame;
    DataManager.setupNewGame      = function() {
        _DataManager_setupNewGame.apply(this, arguments);
        ConfigManager.exportCustomParams();
    };

    var _DataManager_loadGameWithoutRescue = DataManager.loadGameWithoutRescue;
    DataManager.loadGameWithoutRescue      = function(savefileId) {
        var result = _DataManager_loadGameWithoutRescue.apply(this, arguments);
        ConfigManager.importCustomParams(); // white_esorat独自追記　ロード時にはセーブデータのコンフィグ内容を優先する。
        // ConfigManager.exportCustomParams(); // white_esorat独自削除　ロード時にはセーブデータのコンフィグ内容を維持する。
        return result;
    };

    //=============================================================================
    // Window_Options
    //  追加項目を描画します。
    //=============================================================================
    var _Window_Options_initialize      = Window_Options.prototype.initialize;
    Window_Options.prototype.initialize = function() {
        this._customParams = this._customParams || {};
        
        // ウィンドウ移動演出
        this._windowPositionDirection = this._windowPositionDirection || 4; // ウィンドウ位置　4:左, 6:右
        this._windowMoveInOut = true;
        this._windowMoveObject = Esrt.WindowMove.Option;
        
        _Window_Options_initialize.apply(this, arguments);
        this.select(localOptionWindowIndex);
        localOptionWindowIndex = 0;
        this._canProcessTouchNonActive = true;
    };
    var _Window_Options_makeCommandList      = Window_Options.prototype.makeCommandList;
    Window_Options.prototype.makeCommandList = function() {
        _Window_Options_makeCommandList.apply(this, arguments);
        this.addCustomOptions();
    };

    Window_Options.prototype.addCustomOptions = function() {
        iterate(this._customParams, function(key, item) {
            if (!ConfigManager.hiddenInfo[key]) this.addCommand(item.name, key);
        }.bind(this));
    };

    var _Window_Options_statusText      = Window_Options.prototype.statusText;
    Window_Options.prototype.statusText = function(index) {
        var result = _Window_Options_statusText.apply(this, arguments);
        var symbol = this.commandSymbol(index);
        var value  = this.getConfigValue(symbol);
        if (this.isNumberSymbol(symbol)) {
            if (symbol.contains("Number1")) {
                result = this.numberStatusTextEx(symbol,value);
            }else{
                result = this.numberStatusText(value);
            }
        } else if (this.isStringSymbol(symbol)) {
            result = this.stringStatusText(value, symbol);
        }
        return result;
    };

    Window_Options.prototype.isNumberSymbol = function(symbol) {
        return symbol.contains(ConfigManager._symbolNumber);
    };

    Window_Options.prototype.isStringSymbol = function(symbol) {
        return symbol.contains(ConfigManager._symbolString);
    };

    Window_Options.prototype.isCustomSymbol = function(symbol) {
        return !!this._customParams[symbol];
    };

    Window_Options.prototype.numberStatusText = function(value) {
        return value;
    };
    Window_Options.prototype.numberStatusTextEx = function(symbol, value) { //white_esorat追記
        if (symbol === "Number1" && value === 6){
            return "∞";
        }else{
            return value;
        }
    };

    Window_Options.prototype.stringStatusText = function(value, symbol) {
        return this._customParams[symbol].values[value];
    };

    var _Window_Options_processOk      = Window_Options.prototype.processOk;
    Window_Options.prototype.processOk = function() {
        if (!this._shiftValue(1, true)) _Window_Options_processOk.apply(this, arguments);
        this.execScript();
    };

    var _Window_Options_cursorRight      = Window_Options.prototype.cursorRight;
    Window_Options.prototype.cursorRight = function(wrap) {
        if (!this._shiftValue(1, false)) _Window_Options_cursorRight.apply(this, arguments);
    };

    var _Window_Options_cursorLeft      = Window_Options.prototype.cursorLeft;
    Window_Options.prototype.cursorLeft = function(wrap) {
        if (!this._shiftValue(-1, false)) _Window_Options_cursorLeft.apply(this, arguments);
    };

    Window_Options.prototype._shiftValue = function(sign, loopFlg) {
        var symbol = this.commandSymbol(this.index());
        var value  = this.getConfigValue(symbol);
        if (this.isNumberSymbol(symbol)) {
            value += this.numberOffset(symbol) * sign;
            this.changeValue(symbol, this._clampValue(value, symbol, loopFlg));
            return true;
        }
        if (this.isStringSymbol(symbol)) {
            value += sign;
            this.changeValue(symbol, this._clampValue(value, symbol, loopFlg));
            return true;
        }
        return false;
    };

    Window_Options.prototype.execScript = function() {
        var symbol = this.commandSymbol(this.index());
        if (!this.isCustomSymbol(symbol)) return;
        var script = this._customParams[symbol].script;
        if (script) eval(script);
        localOptionWindowIndex = this.index();
    };

    Window_Options.prototype._clampValue = function(value, symbol, loopFlg) {
        var maxValue = this._customParams[symbol].max;
        var minValue = this._customParams[symbol].min;
        if (loopFlg) {
            if (value > maxValue) value = minValue;
            if (value < minValue) value = maxValue;
        }
        return value.clamp(this._customParams[symbol].min, this._customParams[symbol].max);
    };

    Window_Options.prototype.numberOffset = function(symbol) {
        var value = this._customParams[symbol].offset;
        if (Input.isPressed('shift')) value *= 10;
        return value;
    };

    Window_Options.prototype.windowHeight = function() {
        return this.fittingHeight(Math.min(this.numVisibleRows(), 14));
    };
    
    Window_Options.prototype.cursorDown = function(wrap) {
        var index = this.index();
        var maxItems = this.maxItems();
        var maxCols = this.maxCols();
        if (index < maxItems - maxCols) {
            this.select(index + maxCols);
        }else{
            Input.clear();
            this.cursorDownChangeWindow();
        }
    };
    
    
    Window_Options.prototype.cursorUp = function(wrap) {
        var index = this.index();
        var maxItems = this.maxItems();
        var maxCols = this.maxCols();
        if (index >= maxCols) {
            this.select(index - maxCols);
        }else{
            Input.clear();
            this.cursorUpChangeWindow();
        }
    };
    
    Window_Options.prototype.cursorDownChangeWindow = function(wrap) {
        this._scene.activateOption2();
        this._scene._optionsWindow2.select(0);
    };
    
    Window_Options.prototype.cursorUpChangeWindow = function(wrap) {
        if (this._scene._isAvailableOption4) {
            this._scene.activateOption4();
            this._scene._optionsWindow4.select( this._scene._optionsWindow4.maxItems()-1 );
        } else {
            this._scene.activateOption3();
            this._scene._optionsWindow3.select( this._scene._optionsWindow3.maxItems()-1 );
        }
    };
    
    //=============================================================================
    // Window_Options2
    //  white_esorat独自追記 2つ目のオプションウィンドウを設定します
    //=============================================================================    
    Window_Options2.prototype             = Object.create(Window_Options.prototype);
    Window_Options2.prototype.constructor = Window_Options2;
    var _Window_Options2_initialize      = Window_Options2.prototype.initialize;
    Window_Options2.prototype.initialize = function() {
        this._customParams = ConfigManager.getCustomParams1();
        
        // ウィンドウ移動演出
        this._windowPositionDirection = 4; // ウィンドウ位置　4:左, 6:右
        
        _Window_Options2_initialize.apply(this, arguments);
        this.select(0);
    };
    
    Window_Options2.prototype.makeCommandList = function() {
        this.addCustomOptions();
    };
    
    Window_Options2.prototype.cursorDownChangeWindow = function(wrap) {
        this._scene.activateOption3();
        this._scene._optionsWindow3.select(0);
    };
    
    Window_Options2.prototype.cursorUpChangeWindow = function(wrap) {
        this._scene.activateOption();
        this._scene._optionsWindow.select( this._scene._optionsWindow.maxItems()-1 );
    };
    
    
    Window_Options2.prototype.select = function(wrap) {
        Window_Options.prototype.select.apply(this,arguments);
    };
    
    //=============================================================================
    // Window_Options3
    //  white_esorat独自追記 3つ目のオプションウィンドウを設定します
    //=============================================================================    
    Window_Options3.prototype             = Object.create(Window_Options.prototype);
    Window_Options3.prototype.constructor = Window_Options3;
    var _Window_Options3_initialize      = Window_Options3.prototype.initialize;
    Window_Options3.prototype.initialize = function() {
        this._customParams = ConfigManager.getCustomParams2();
        
        // ウィンドウ移動演出
        this._windowPositionDirection = 6; // ウィンドウ位置　4:左, 6:右
        
        _Window_Options3_initialize.apply(this, arguments);
        this.select(0);
    };
    
    Window_Options3.prototype.makeCommandList = function() {
        this.addCustomOptions();
    };
    
    Window_Options3.prototype.cursorDownChangeWindow = function(wrap) {
        if (this._scene._isAvailableOption4) {
            this._scene.activateOption4();
            this._scene._optionsWindow4.select(0);
        } else {
            this._scene.activateOption();
            this._scene._optionsWindow.select(0);
        }
    };
    
    Window_Options3.prototype.cursorUpChangeWindow = function(wrap) {
        this._scene.activateOption2();
        this._scene._optionsWindow2.select( this._scene._optionsWindow2.maxItems()-1 );
    };
    
    //=============================================================================
    // Window_Options4
    //  white_esorat独自追記 3つ目のオプションウィンドウを設定します
    //=============================================================================    
    Window_Options4.prototype             = Object.create(Window_Options.prototype);
    Window_Options4.prototype.constructor = Window_Options4;
    var _Window_Options4_initialize      = Window_Options4.prototype.initialize;
    Window_Options4.prototype.initialize = function() {
        this._customParams = ConfigManager.getCustomParams3();
        
        // ウィンドウ移動演出
        this._windowPositionDirection = 6; // ウィンドウ位置　4:左, 6:右
        
        _Window_Options4_initialize.apply(this, arguments);
        this.select(0);
    };
    
    Window_Options4.prototype.windowHeight = function() {
        return this.fittingHeight(7);
    };
    
    Window_Options4.prototype.makeCommandList = function() {
        this.addCustomOptions();
    };
    
    Window_Options4.prototype.cursorDownChangeWindow = function(wrap) {
        this._scene.activateOption();
        this._scene._optionsWindow.select(0);
    };
    
    Window_Options4.prototype.cursorUpChangeWindow = function(wrap) {
        this._scene.activateOption3();
        this._scene._optionsWindow3.select( this._scene._optionsWindow3.maxItems()-1 );
    };
    //=============================================================================
    // Window_Selectable
    //  ウィンドウ切り替え時に選択のみ行うようにする
    //=============================================================================  
    
    Window_Selectable.prototype.onTouchSelectOnly = function(triggered) {
        var lastIndex = this.index();
        var x = this.canvasToLocalX(TouchInput.x);
        var y = this.canvasToLocalY(TouchInput.y);
        var hitIndex = this.hitTest(x, y);
        if (hitIndex >= 0) {
            this.select(hitIndex);
        } else if (this._stayCount >= 10) {
            if (y < this.padding) {
                this.cursorUp();
            } else if (y >= this.height - this.padding) {
                this.cursorDown();
            }
        }
        if (this.index() !== lastIndex) {
            SoundManager.playCursor();
        }
    };
})();
