//=============================================================================
// ESRT_EventHistory.js
// ----------------------------------------------------------------------------
// Copyright (c) 2019 @white_esorat
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// [HP]   : https://esora-t.jp/
// [Twitter]: https://twitter.com/white_esorat
//=============================================================================

/*:
 * @plugindesc イベントの実行履歴を記録し、指定したスイッチ・変数の状態を再現して特定のイベントを回想する
 * @author 白
 *
 * @param 記録情報
 *
 * @param 履歴記録スイッチ
 * @parent 記録情報
 * @desc カンマ区切りで複数指定、ハイフンで範囲指定が可能です。
 * (例) 1-5,15,17,21-23,40,1020
 * @default
 *
 * @param 履歴記録変数
 * @parent 記録情報
 * @desc カンマ区切りで複数指定、ハイフンで範囲指定が可能です。
 * (例) 1-5,15,17,21-23,40,1020
 * @default
 *
 *
 * @param 上部タイトル情報
 *
 * @param 上部タイトル
 * @parent 上部タイトル情報
 * @desc 上部ウィンドウのタイトルです。
 * @default イベント履歴一覧
 * @type string
 *
 * @param 一覧表示内容
 *
 * @param 一覧表示テキスト
 * @parent 一覧表示内容
 * @desc 一覧に表示する文字列です。一部文字列は変換して表示されます。（半角スペースは無視されます）
 * @default [group] subject action
 * @type string
 *
 * @param 一覧変換テキスト
 * @parent 一覧表示内容
 * @desc 一覧に表示する文字列のうち、指定した文字列を特定のスイッチ・変数の値、もしくは値に対応した内容で置き換えます。
 * @default
 * @type struct<TranslationData>[]
 *
 * @param 状態表示内容
 *
 * @param 下部タイトル
 * @parent 状態表示内容
 * @desc 下部ウィンドウのタイトルです。
 * @default 当時の状態
 * @type string
 *
 * @param 下部タイトル表示位置
 * @parent 状態表示内容
 * @desc 下部ウィンドウのタイトル位置です。
 * @default 当時の状態
 * @type select
 *
 * @option 左
 * @value 1
 *
 * @option 中央
 * @value 2
 *
 * @option 右
 * @value 3
 *
 * @default 1
 *
 * @param 表示情報
 * @parent 状態表示内容
 * @desc 下部ウィンドウに表示する情報です。
 * @default
 * @type struct<StatusData>[]
 *
 * @help
 * ■ dataフォルダに用意するファイル
 *   GalleryList.json       ： 回想イベント呼び出し用データ
 *   EventHistoryData.json  ： イベント履歴記録用データ
 *
 * ■ プラグインパラメータ
 *   履歴記録スイッチ
 *     イベント履歴に表示、もしくは再現時に仕様するスイッチの番号を指定します。
 *     カンマ区切りで複数指定、ハイフンで範囲指定が可能です。
 *     例） 1-5,15,17,21-23,40,1020
 *       => 1,2,3,4,5,15,17,21,22,23,40,1020番のスイッチの状態を履歴記録時に保存
 *
 *   履歴記録変数
 *     イベント履歴に表示、もしくは再現時に仕様する変数の番号を指定します。
 *     カンマ区切りで複数指定、ハイフンで範囲指定が可能です。
 *     例） 1-5,15,17,21-23,40,1020
 *       => 1-5,15,17,21-23,40,1020番の変数の状態を履歴記録時に保存
 *
 *   一覧表示テキスト
 *     イベント履歴一覧に表示する文字列です。
 *    「group」「subject」「action」の文字列は、
 *     それぞれ「EventHistoryData.json」で設定した内容に変換して表示されます。
 *    （半角スペースは無視されます）
 *     例） イベント履歴に記録されているデータが
 *          group=1、subject=2、action=3のとき、
 *         「EventHistoryData.json」で入力した内容に従って変換されます。
 *              [group] subject action
 *            => [食事] 二人でミルクティーを飲んだ
 *
 *
 *   一覧変換テキスト
 *     一覧表示テキストのうち、「group」「subject」「action」以外にも
 *     追加での文字列の変換を行うようにします。
 *     例） スイッチ番号 10がONのとき、（満腹）と表示させたいとき
 *           Name            manpuku
 *           TypeSelect      1  (プルダウンで「スイッチ」を選択)
 *           SwitchID        10
 *           DisplayStrings  ["","(満腹)"]
 *        と設定し、「一覧表示テキスト」を以下のように設定する。
 *              [group] manpuku subject action
 *            => [食事] (満腹)二人でミルクティーを飲んだ
 *
 *   表示情報
 *     下部ウィンドウに表示する情報です。入力した内容が順番に
 *       1     2
 *       3     4
 *       5     6
 *       7     8
 *     と言う順番で表示されます。
 *     TypeSelectでスイッチか変数を選択し、SwitchIDかVariableIDで番号を指定すると、
 *     その番号の値、もしくは値に対応したDisplayStringsの内容を表示します。
 *     例）  Name            満腹度
 *           TypeSelect      2  (プルダウンで「変数」を選択)
 *           VariableID      20
 *           DisplayStrings  ["空腹", "そこそこ", "満足"]
 *             => イベント履歴記録時の変数20が
 *                   0のとき：空腹と表示
 *                   1のとき：そこそこと表示
 *                   2のとき：満足と表示
 *
 * ■ プラグインコマンド
 *  ・イベント履歴の保存
 *
 *    例）EventHistory setEvent [groups（分類）] [subjects（相手orイベント名）] [actions（内容）] [eventData（回想内容）]
 *
 *  イベントコマンド「プラグインコマンド」から実行します。
 *  （パラメータの間は半角スペースで区切ります）
 *  最初の三つのパラメータは順番に「EventHistoryData.json」groups、subjects、actionsの内容に対応した情報を保存します。
 *  最後のパラメータは「GalleryList.json」のeventDataに対応した情報を保存します。
 *  （※セーブデータ内では最初の三つのパラメータは数値として、最後のパラメータは文字列として記録します。
 *      セーブデータに保存後、後から「EventHistoryData.json」「GalleryList.json」の内容を変更した場合、
 *      保存した情報に対応する変更後の「EventHistoryData.json」「GalleryList.json」の内容を使用します）
 *
 *  最初の三つのパラメータには以下の三種類の指定方法があります。
 *    数値を入力：数字がそのまま保存されます
 *    文字を入力：文字に対応した数値の情報が保存されます
 *    v[n]と入力：変数番号nの値を使用します。
 *  例1）EventHistoryData.jsonのactionsの項目が
 *    "actions" : {
 *            "1" : {"short":"ハンバーガー", "text": "ハンバーガーを食べる"}
 *    }
 *    のとき、
 *    EventHistory setEvent 食事 一人 ハンバーガーを食べる 1-1-1
 *    EventHistory setEvent 食事 一人 ハンバーガー 1-1-1
 *    EventHistory setEvent 食事 一人 1 1-1-1
 *    は同じ保存内容となります。
 *   （「ハンバーガーを食べる」「ハンバーガー」という文字が1に変換されます。
 *     「text」で設定した文字列はイベント一覧で表示される文字列、
 *     「short」で設定した文字列はプラグインコマンド入力時に短く入力するための文字列です。
 *     短い文字列が必要ないときは、「short」は設定しなくても「text」だけで動作はします）
 *  例2） EventHistory setEvent 食事 一人 v[1] 1-1-1
 *    とすることでactionsとして保存する内容が変数の中身の数値になります
 *
 */
/*~struct~StatusData:
 * @param Name
 * @desc 変換元の文字列です。
 * @default data1
 *
 * @param TypeSelect
 * @desc スイッチ・変数のどちらを表示するか選択します。（選ばれなかった項目への入力は無視されます）
 * @type select
 *
 * @option スイッチ
 * @value 1
 *
 * @option 変数
 * @value 2
 *
 * @default 1
 *
 * @param SwitchID
 * @desc 項目の設定内容が格納される変数番号です。
 * @default 0
 * @type switch
 *
 * @param VariableID
 * @desc 項目の設定内容が格納される変数番号です。
 * @default 0
 * @type variable
 *
 * @param DisplayStrings
 * @desc 表示用文字列の配列です。入力されている場合、スイッチ・変数の状態に対応した文字列が表示されます。
 * @default
 * @type string[]
 */

/*~struct~TranslationData:
 * @param Name
 * @desc 項目の名称です。
 * @default 情報1
 *
 * @param TypeSelect
 * @desc スイッチ・変数のどちらを表示するか選択します。（選ばれなかった項目への入力は無視されます）
 * @type select
 *
 * @option スイッチ
 * @value 1
 *
 * @option 変数
 * @value 2
 *
 * @option 空白
 * @value 3
 *
 * @default 1
 *
 * @param SwitchID
 * @desc 項目の設定内容が格納される変数番号です。
 * @default 0
 * @type switch
 *
 * @param VariableID
 * @desc 項目の設定内容が格納される変数番号です。
 * @default 0
 * @type variable
 *
 * @param DisplayStrings
 * @desc 表示用文字列の配列です。入力されている場合、スイッチ・変数の状態に対応した文字列が表示されます。
 * @default
 * @type string[]
 */

var Esrt = Esrt || {};
Esrt.eventHistory   = Esrt.eventHistory || [];
Esrt.eventHistoryIndex   = Esrt.eventHistoryIndex || [];
Esrt.eventHistoryIndex.topRow = 0;
Esrt.eventHistoryIndex.index = 0;

Esrt.eventHistoryReverse = {};
Esrt.eventHistoryReverse.group = null;
Esrt.eventHistoryReverse.subject = null;
Esrt.eventHistoryReverse.action = null;

function Scene_EventHistory() {
    this.initialize.apply(this, arguments);
}

function Window_EventHistory() {
    this.initialize.apply(this, arguments);
}

function Window_EventHistoryStatus() {
    this.initialize.apply(this, arguments);
}

function Window_EventHistoryTitle() {
    this.initialize.apply(this, arguments);
}

(function() {
    'use strict';
    var pluginName    = 'ESRT_EventHistory';

    //=============================================================================
    // パラメータの取得と整形
    //=============================================================================
    var parameters = PluginManager.parameters(pluginName);
    var menuShowSwitch     = 211;
    var menuAllowSwitch    = menuShowSwitch + 6;

    //=============================================================================
    // DataManager
    //  イベント履歴定義ファイル読み込み処理を定義します
    //=============================================================================
    DataManager.loadEventHistoryData = function() {
        this.loadDataFile('$dataEventHistory', 'EventHistoryData.json');
        return;
    };

    //=============================================================================
    // Scene_Boot
    //  イベント履歴定義ファイル読み込み処理を追加定義します
    //=============================================================================
    var _Scene_Boot_create      = Scene_Boot.prototype.create;
    Scene_Boot.prototype.create = function() {
        _Scene_Boot_create.call(this);
        DataManager.loadEventHistoryData();
    };

    //=============================================================================
    // Game_Interpreter
    //  プラグインコマンドを追加定義します
    //=============================================================================
    var _Game_Interpreter_pluginCommand      = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        _Game_Interpreter_pluginCommand.apply(this, arguments);
        if(command === 'EventHistory'){
            switch(args[0]){
                case 'setEvent':
                    SetEvent(args);
                    break;
                default:
                    break;
            }
        }
    };

    // イベント履歴の記録
    function SetEvent(args) {
        if (Esrt.gallery.typeSelected >= 1) {return;} // 回想時には記録しない
        MakeDataReverse();

        var regVar = /v\[(\d+)\]/;
        var groupId   = GetIndexId(args[1], Esrt.eventHistoryReverse.group);
        var subjectId = GetIndexId(args[2], Esrt.eventHistoryReverse.subject);
        var actionId  = GetIndexId(args[3], Esrt.eventHistoryReverse.action);

        var eventKey = (typeof args[4] != "undefined") ? args[4] : "";

        var statusSwitch   = GetStatusSwitchText();
        var statusVariable = GetStatusVariableText();

        Esrt.eventHistory.push(groupId + "/" + subjectId + "/" + actionId + "/" + eventKey + "/" + statusSwitch + statusVariable);
    }

    // プラグインコマンドの入力を解析し、対応する番号に変換する
    //   v[〇〇]　-> 変数の値を代入
    //   文字列   -> 逆引きデータから得た値を代入
    function GetIndexId(text, reverseData) {
        var regVar = /v\[(\d+)\]/;
        var id = text;

        if (regVar.test(id)) { // 変数が指定されているとき、入力を解析して変数の値を代入する
            id = Number(id.replace(regVar, $gameVariables.value(RegExp.$1)));
        }

        if (isNaN(id)) { // 入力が数値でないとき、文字から逆引きした番号を代入する
            id = reverseData[id];
            if (typeof id === "undefined") {
                id = 0
            }
        }

        return id;
    }

    function GetStatusSwitchText() {
        var text = "";
        var switches = GetNumberArray(parameters["履歴記録スイッチ"]);

        switches.forEach( function(swith) {
            var value = ($gameSwitches.value(swith)) ? 1 : 0;
            text += "s" + swith + ":" + value + "/";
        });

        return text;
    }

    function GetStatusVariableText() {
        var text = "";
        var variables = GetNumberArray(parameters["履歴記録変数"]);

        variables.forEach( function(variable) {
            text += "v" + variable + ":" + $gameVariables.value(variable) + "/";
        });

        return text;
    }

    function GetNumberArray(parameter) {
        var return_array = [];
        var split = ",";
        var range_split = "-";

        var texts = parameter.split(split);

        for (var i=0; i < texts.length; i++) {
            var regExp = /\d+-\d+/;

            if (regExp.test(texts[i])) { // 範囲指定記録
                var range = texts[i].split(range_split, -1);
                if (isNaN(Number(range[0])) || isNaN(Number(range[1])) ) { continue; }

                for (var j = Number(range[0]); j <= Number(range[1]); j++) {
                    return_array.push(j);
                }

            } else { // 数値単体記録
                var number = Number(texts[i]);

                if (isNaN(number)) { continue;}

                return_array.push(number);
            }
        }

        return return_array;
    }

    // EventHistoryData.jsonのデータから逆引き情報を作成する
    function MakeDataReverse() {
        if(Esrt.eventHistoryReverse.group && !$dataEventHistory) {return}; // 既に逆引き情報を作成しているとき、元のデータが読み込めていないときは実行しない

        Esrt.eventHistoryReverse.group = {};
        Esrt.eventHistoryReverse.subject = {};
        Esrt.eventHistoryReverse.action = {};

        Object.keys($dataEventHistory.groups).forEach(function (key) {
            var data = $dataEventHistory.groups[key];
            var text = (data.short) ? data.short : data.text;

            Esrt.eventHistoryReverse.group[text] = key;
        });

        Object.keys($dataEventHistory.subjects).forEach(function (key) {
            var data = $dataEventHistory.subjects[key];
            var text = (data.short) ? data.short : data.text;

            Esrt.eventHistoryReverse.subject[text] = key;
        });

        Object.keys($dataEventHistory.actions).forEach(function (key) {
            var data = $dataEventHistory.actions[key];
            var text = (data.short) ? data.short : data.text;

            Esrt.eventHistoryReverse.action[text] = key;
        });

        return;
    }

    function parseStructStringToJSON(string) {
        var json = [];

        try {
            json = JSON.parse(string);

            if (json === null) {
                json = [];
                return;
            }

            json = json.map(function(textData) {
                var data = JSON.parse(textData);
                data.DisplayStrings = (data.DisplayStrings) ? JSON.parse(data.DisplayStrings) : [];

                return data;
            });

        } catch (e) {
            alert(`Error : Plugin parameter!\nPlugin:${pluginName}.js\nName:["表示情報"]\nValue:${statuses}`);
            json = [];
        }

        return json;
    }

    function getTextFromSwitchVariable(parameter, switches, variables) {
        // 表示データとしてスイッチ・変数のどちらを表示するか判定
        var sorceData = [];
        var sorceNumber = 0;

        if (parameter.TypeSelect == 1) {
            sorceData = switches;
            sorceNumber = parameter.SwitchID;

        } else if (parameter.TypeSelect == 2) {
            sorceData = variables;
            sorceNumber = parameter.VariableID;
        }

        if (typeof sorceData[sorceNumber] === "undefined") {return;}

        // 表示データの取得
        var text = parameter.DisplayStrings[sorceData[sorceNumber]];

        if (typeof text === "undefined") {
            text = sorceData[sorceNumber];
        };

        return text;
    }

    //=============================================================================
    // DataManager
    //  イベント履歴をセーブに含めます
    //=============================================================================
    var _DataManager_makeSaveContents = DataManager.makeSaveContents;
        DataManager.makeSaveContents = function() {
        var contents = _DataManager_makeSaveContents.call(this);
        contents.eventHistory = Esrt.eventHistory;
        return contents;
    };
    var _DataManager_extractSaveContents = DataManager.extractSaveContents;
        DataManager.extractSaveContents = function(contents) {
        _DataManager_extractSaveContents.call(this, contents);
        Esrt.eventHistory = contents.eventHistory;
        if(typeof Esrt.eventHistory === "undefined") Esrt.eventHistory = [];
    };

    //=============================================================================
    // Scene_EventHistory
    //  イベント履歴表示画面を扱うクラス
    //=============================================================================

    Scene_EventHistory.prototype = Object.create(Scene_MenuBase.prototype);
    Scene_EventHistory.prototype.constructor = Scene_EventHistory;

    Scene_EventHistory.prototype.initialize = function() {
        MakeDataReverse();

        this._startFlag = false;

        Scene_MenuBase.prototype.initialize.call(this);
        this._touchInterval = 0;
        Esrt.gallery.typeSelected = 0;
    };

    Scene_EventHistory.prototype.update = function() {
        Scene_MenuBase.prototype.update.call(this);
        if(this._touchInterval > 0){
            this._touchInterval -= 1;
        }
        this.updateBackgroundEffects();
    };

    Scene_EventHistory.prototype.updateBackgroundEffects = function() {
        this._backgroundTile.origin.x += 0.5;
        this._backgroundTile.origin.y += 0.5;
    };

    Scene_EventHistory.prototype.create = function() {
        Scene_MenuBase.prototype.create.call(this);
        this.createHistoryWindow();
        this.createStatusWindow();
        this.createTitleWindow();
    };

    Scene_EventHistory.prototype.createHistoryWindow = function() {
        this._historyWindow = new Window_EventHistory(0, 65);
        this._historyWindow.setHandler('ok', this.commandPlayGallery.bind(this));
        this._historyWindow.setHandler('cancel', this.commandBack.bind(this));
        this.addWindow(this._historyWindow);
    }

    Scene_EventHistory.prototype.createStatusWindow = function() {
        this._statusWindow = new Window_EventHistoryStatus(0, 340);
        this.addWindow(this._statusWindow);
        this._historyWindow.setStatusWindow(this._statusWindow);
    }

    Scene_EventHistory.prototype.createTitleWindow = function() {
        this._titleWindow = new Window_EventHistoryTitle(0, 0);
        this.addWindow(this._titleWindow);
    }

    Scene_EventHistory.prototype.createBackground = function() {
        var width = Graphics.boxWidth * 2;
        var height = Graphics.boxHeight * 2;

        this._backgroundSprite = new Sprite();
        this.addChild(this._backgroundSprite);

        this._backgroundTile = new TilingSprite(ImageManager.loadTitle1("Background_3"));
        this._backgroundTile.move(0,0, width, height);
        this._backgroundTile.anchor.x = 0.5;
        this._backgroundTile.anchor.y = 0.5;
        this._backgroundSprite.addChild(this._backgroundTile);
    };

    Scene_EventHistory.prototype.backScene = function() {
        if(this._startFlag === true || this._touchInterval > 0 ){ // 1クリックで複数呼び出されることに対する対処
            return;
        }
        SoundManager.playCancel();
        this.popScene();
    }

    Scene_EventHistory.prototype.commandBack = function() {
        if(this._startFlag === true || this._touchInterval > 0 ){ // 1クリックで複数呼び出されることに対する対処
            return;
        }
        this._touchInterval = 15;
        SoundManager.playCancel();
        this.popScene();
    };

    Scene_EventHistory.prototype.commandPlayGallery = function() {
        if(this._startFlag === true ){ // 複数呼び出されることに対する対処
            return;
        }

        var item = this._historyWindow.item();

        if (!item || !item.eventData) {
            SoundManager.playBuzzer();
            return;
        }

        this._startFlag = true;
        Esrt.eventHistoryIndex.topRow = this._historyWindow.topRow();
        Esrt.eventHistoryIndex.index = this._historyWindow.index();
        Esrt.gallery.isFromGallery = true;
        Esrt.gallery.isCGMode = 0;
        Esrt.gallery.typeSelected  = 3; // 回想閲覧後に回想画面に戻ったとき、直接イベント履歴画面に戻るためのフラグ

        for(var i=0; i <= 100; i++){$gameScreen.erasePicture(i)}

        $gamePlayer.setTransparent(255);
        this.fadeOutAll();

        this.setData(item);

        $gamePlayer.reserveTransfer(7, 0, 0, 0);
        this.popScene();
        this.popScene();

        this._historyWindow.activate();
    };

    Scene_EventHistory.prototype.setData = function(item) {
        if(item.statusSwitches){ // 履歴に記録されたスイッチの値を適用
            Object.keys(item.statusSwitches).forEach(function(key) {
                var value = (Number(item.statusSwitches[key]) === 1) ? true : false;
                $gameVariables.setValue(key, value);
            });
        }

        if(item.statusVariables){ // 履歴に記録された変数の値を適用
            Object.keys(item.statusVariables).forEach(function(key) {
                $gameVariables.setValue(key, Number(item.statusVariables[key]));
            });
        }

        if(item.eventData.setVariables){ // 回想データで指定した変数の値を適用
            Object.keys(item.eventData.setVariables).forEach(function(key) {
                $gameVariables.setValue(key, item.eventData.setVariables[key]);
            });
        }

    };

    //=============================================================================
    // Window_EventHistory
    //  イベント履歴一覧ウィンドウ
    //=============================================================================
    Window_EventHistory.prototype = Object.create(Window_Selectable.prototype);
    Window_EventHistory.prototype.constructor = Window_EventHistory;

    Window_EventHistory.prototype.initialize = function(x, y) {
        var width = Graphics.boxWidth;
        var height = this.fittingHeight(7);

        Window_Selectable.prototype.initialize.call(this, x, y, width, height);

        this.refresh();
        this.setTopRow(Esrt.eventHistoryIndex.topRow);
        this.select(Esrt.eventHistoryIndex.index);
        this.activate();
    };

    Window_EventHistory.prototype.maxCols = function() {
        return 1;
    };

    Window_EventHistory.prototype.maxItems = function() {
        return this._data ? this._data.length : 0;
    };

    var _Window_EventHistory_select = Window_EventHistory.prototype.select;
    Window_EventHistory.prototype.select = function(index) {
        _Window_EventHistory_select.apply(this, arguments);
        this.updateStatus();
    };

    Window_EventHistory.prototype.setStatusWindow = function(setWindow) {
        this._statusWindow = setWindow;
        this.updateStatus();
    };

    Window_EventHistory.prototype.updateStatus = function(index) {
        if (this._statusWindow) {
            this._statusWindow.setStatus(this.item());
        }
    };

    Window_EventHistory.prototype.refresh = function() {
        this.makeEventList();
        this.createContents();
        this.drawAllItems();
    };

    Window_EventHistory.prototype.makeEventList = function() {
        this._data = [];

        for (var j = 0; j < Esrt.eventHistory.length; j++) {
            var data = Esrt.eventHistory[j];
            var dataArray = data.split("/");

            var pushItem = {};

            this.getEventData(pushItem, dataArray);
            this.getStatusData(pushItem, dataArray);

            var text =  parameters["一覧表示テキスト"];

            text = text.replace(/\s+/g, "");
            text = this.translateText(text, dataArray);
            text = this.translateEx(text, dataArray, pushItem.statusSwitches, pushItem.statusVariables);

            pushItem.text = text;

            this._data.push(pushItem);
        }
    }

    Window_EventHistory.prototype.getEventData = function(pushItem, dataArray) {

        pushItem.eventData = null;
        if (dataArray.length > 4) {
            var key = dataArray[3];
            if (key.match(/-/) && $dataGallery.eventData[key]) {
                pushItem.eventData = $dataGallery.eventData[key];
            }
        }
    }

    Window_EventHistory.prototype.getStatusData = function(pushItem, dataArray) {
        pushItem.statusSwitches = {};
        pushItem.statusVariables = {};

        if (dataArray.length > 5) {

            // イベント実行時のスイッチ・変数を適用
            for (var i=4; i<dataArray.length; i++) {
                if (dataArray[i].match(/:/)) {
                    var statusData = dataArray[i].split(":");
                    var setObject = pushItem.statusVariables
                    if (statusData[0].slice( 0, 1 ) === "s") {
                        setObject = pushItem.statusSwitches;
                        statusData[0] = statusData[0].slice( 1 );

                    } else if (statusData[0].slice( 0, 1 ) === "v") {
                        statusData[0] = statusData[0].slice( 1 );

                    }
                    setObject[statusData[0]] = statusData[1];
                }
            }
        }
    }

    Window_EventHistory.prototype.translateText = function(text, dataArray) {
        text = this.translateGroupText(text, dataArray);
        text = this.translateSubjectText(text, dataArray);
        text = this.translateActionText(text, dataArray);

        return text;
    }

    Window_EventHistory.prototype.translateGroupText = function(text, dataArray) {
        var groupText   = ($dataEventHistory.groups[dataArray[0]])   ? $dataEventHistory.groups[dataArray[0]].text   : "---";
        text = text.replace("group", groupText);

        return text;
    }

    Window_EventHistory.prototype.translateSubjectText = function(text, dataArray) {
        var subjectText = ($dataEventHistory.subjects[dataArray[1]]) ? $dataEventHistory.subjects[dataArray[1]].text : "---";
        text = text.replace("subject", subjectText);

        return text;
    }

    Window_EventHistory.prototype.translateActionText = function(text, dataArray) {
        var actionText  = ($dataEventHistory.actions[dataArray[2]])  ? $dataEventHistory.actions[dataArray[2]].text  : "---";
        text = text.replace("action", actionText);

        return text;
    }

    Window_EventHistory.prototype.translateEx = function(text, dataArray, switches, variables) {
        var translatitonList = parseStructStringToJSON(parameters["一覧変換テキスト"]);

        translatitonList.forEach( function(data) {
            var translateText = getTextFromSwitchVariable(data, switches, variables);
            text = text.replace(data.Name, translateText);
        });

        return text;
    }

    Window_EventHistory.prototype.item = function() {
        var index = this.index();
        return this._data && index >= 0 ? this._data[index] : null;
    };

    Window_EventHistory.prototype.isEnabled = function(item) {
        if(!item) return false;

        var result = (item.eventData) ? true : false;
        return result;
    };

    Window_EventHistory.prototype.isCurrentItemEnabled = function() {
        return this.isEnabled(this._data[this.index()]);
    };

    Window_EventHistory.prototype.drawItem = function(index) {
        var item = this._data[index];
        var rect = this.itemRectForText(index);
        var name = index + ":" + item.text;

        this.resetTextColor();
        if (!this.isEnabled(item)) {
            this.changeTextColor(this.textColor(8));
        }
        this.drawText(name, rect.x, rect.y, rect.width);
    };

    Window_EventHistory.prototype.processCancel = function() {
        Window_Selectable.prototype.processCancel.call(this);
        Esrt.eventHistoryIndex.topRow = this.topRow();
        Esrt.eventHistoryIndex.index = this.index();
    };

    //=============================================================================
    // Window_EventHistoryStatus
    //  イベント履歴の実行時状態経験表示ウィンドウ
    //=============================================================================
    Window_EventHistoryStatus.prototype = Object.create(Window_Base.prototype);
    Window_EventHistoryStatus.prototype.constructor = Window_EventHistoryStatus;

    Window_EventHistoryStatus.prototype.initialize = function(x, y) {
        var width = Graphics.boxWidth;
        var height = this.fittingHeight(5);
        Window_Base.prototype.initialize.call(this, x, y, width, height);

        this.refresh();
    };

    Window_EventHistoryStatus.prototype.setStatus = function(item) {
        if (item) {
            this._statusSwitches = item.statusSwitches;
            this._statusVariables = item.statusVariables;
        }
        this.refresh();
    };

    Window_EventHistoryStatus.prototype.refresh = function() {
        this.contents.clear();
        this.drawWindow();
    };

    Window_EventHistoryStatus.prototype.drawWindow = function() {
        this.drawStatusTitle();

        if (!this._statusVariables) {return;}

        this.drawAllStatuses();
    }

    Window_EventHistoryStatus.prototype.drawAllStatuses = function() {
        var x = 10;
        var y = 0;
        var i = 0;

        var statuses = this.parseStatusJson();

        y += this.lineHeight();
        statuses.forEach(function(status) {
            this.drawStatus(status, x, y);

            if (i % 2 === 1) {
                x = 10;
                y += this.lineHeight();

            } else {
                x = 410;
            }
            i++;

        }, this);
    };

    Window_EventHistoryStatus.prototype.drawStatus = function(parameter, x, y) {
        if (parameter.TypeSelect == 3) {return;}

        var text = getTextFromSwitchVariable(parameter, this._statusSwitches, this._statusVariables);

        // データ表示
        this.changeTextColor(this.systemColor());
        this.drawText(parameter.Name, x, y, 134);
        this.resetTextColor();
        this.drawText(text, x + 170, y);
    }

    Window_EventHistoryStatus.prototype.drawStatusTitle = function() {
        var title = parameters["下部タイトル"];
        var align = Number(parameters["下部タイトル表示位置"]);
        align = (align == 2) ? "center" : align;
        align = (align == 3) ? "right"  : align;

        this.drawText(title, 10, 0, this.width - 50, align);
    }

    Window_EventHistoryStatus.prototype.parseStatusJson = function() {
        var statuses = [];

        statuses = parseStructStringToJSON(parameters["表示情報"]);

        return statuses;
    }

    //=============================================================================
    // Window_EventHistoryTitle
    //  イベント履歴タイトルウィンドウ
    //=============================================================================
    Window_EventHistoryTitle.prototype = Object.create(Window_Base.prototype);
    Window_EventHistoryTitle.prototype.constructor = Window_EventHistoryTitle;

    Window_EventHistoryTitle.prototype.initialize = function(x, y) {
        var width = Graphics.boxWidth;
        var height = this.fittingHeight(1);
        Window_Base.prototype.initialize.call(this, x, y, width, height);

        this.refresh();
    };

    Window_EventHistoryTitle.prototype.refresh = function() {
        this.drawText(parameters["上部タイトル"], 0, 0, Graphics.boxWidth, 'center');
    };
})();
