//=============================================================================
// Torigoya_SaveCommand.js
//=============================================================================

/*:
 * @plugindesc Add save command in Plugin Command
 * @author ru_shalm
 *
 * @help
 *
 * Plugin Command:
 *   SaveCommand save 1       # save to slot 1
 *   SaveCommand save [1]     # save to slot variables[1]
 *   SaveCommand save last    # save to last accessed file
 *   SaveCommand load 1       # load from slot 1
 *   SaveCommand load [1]     # load from slot variables[1]
 *   SaveCommand load last    # load from last accessed file
 *   SaveCommand remove 1     # remove save file of slot 1
 *   SaveCommand remove [1]   # remove save file of slot variables[1]
 *   SaveCommand remove last  # remove last accessed file
 *
 * (default last accessed file: 1)
 */

/*:ja
 * @plugindesc プラグインコマンドからセーブを実行できるようにします。
 * @author ru_shalm
 *
 * @help
 * イベントコマンドの「プラグインコマンド」を使って、
 * イベント中に自動的にセーブを実行できるようになります。
 *
 * 例えばオートセーブのゲームなどが作れるようになります。
 *
 * ------------------------------------------------------------
 * ■ プラグインコマンド（セーブ系）
 * ------------------------------------------------------------
 *
 * ● スロット1にセーブを実行する
 * SaveCommand save 1
 *
 * ※ 1 の部分の数字を変えると別のスロットにセーブされます
 *
 * ● 変数[1]番のスロットにセーブを実行する
 * SaveCommand save [1]
 *
 * ● 前回ロード/セーブしたスロットにセーブを実行する
 * SaveCommand save last
 *
 * ※ ロード/セーブしていない場合はスロット1になります。
 *
 * ------------------------------------------------------------
 * ■ プラグインコマンド（ロード系）
 * ------------------------------------------------------------
 * ＜注意＞
 * RPGツクールはイベントの途中で
 * セーブデータがロードされることが想定されていません。
 * そのためイベントのタイミングによっては、
 * ロード後のゲームの動作がおかしくなることがあります。
 *
 * ● スロット1からロードを実行する
 * SaveCommand load 1
 *
 * ● 変数[1]番のスロットからロードを実行する
 * SaveCommand load [1]
 *
 * ● 前回ロード/セーブしたスロットからロードを実行する
 * SaveCommand load last
 *
 * ※ ロード/セーブしていない場合はスロット1になります。
 *
 * ● 一番最後にセーブをしたスロットからロードを実行する
 * SaveCommand load latest
 *
 * ※ last ではなく latest です＞＜；
 *
 * ------------------------------------------------------------
 * ■ プラグインコマンド（削除系）
 * ------------------------------------------------------------
 * ＜注意＞
 * セーブデータを削除するコマンドなので取扱注意ですよ！
 *
 * ● スロット1を削除する
 * SaveCommand remove 1
 *
 * ● 変数[1]番のスロットを削除する
 * SaveCommand remove [1]
 *
 * ● 前回ロード/セーブしたスロットを削除する
 * SaveCommand remove last
 *
 * ※ ロード/セーブしていない場合はスロット1になります。
 */

var Esrt = Esrt || {};
Esrt.autoSave = Esrt.autoSave || {};
Esrt.autoSave.isAutoSave    = false;
Esrt.autoSave.isShow        = false;
Esrt.autoSave.isSaveSuccess = true;

(function (global) {
    var SaveCommand = {};

    SaveCommand.parseSlotId = function (str) {
        var slotId, matches;
        if (matches = str.match(/^\[(\d+)\]$/)) {
            slotId = $gameVariables.value(~~matches[1]);
        } else if (str.match(/^\d+$/)) {
            slotId = ~~str;
        } else {
            switch (str) {
                case 'last':
                    slotId = DataManager.lastAccessedSavefileId();
                    break;
                case 'latest':
                    slotId = DataManager.latestSavefileId();
                    break;
            }
        }

        if (~~slotId <= 0) {
            throw '[Torigoya_SaveCommand.js] invalid SlotId: ' + slotId;
        }

        return slotId;
    };

    SaveCommand.runCommand = function (gameInterpreter, type, slotId) {
        switch (type) {
            case 'load':
                this.runCommandLoad(gameInterpreter, slotId);
                break;
            case 'save':
                this.runCommandSave(gameInterpreter, slotId);
                break;
            case 'remove':
                this.runCommandRemove(gameInterpreter, slotId);
                break;
        }
    };

    // 無理やり感ある
    SaveCommand.runCommandLoad = function (gameInterpreter, slotId) {
        if (!DataManager.isThisGameFile(slotId)) return;

        var scene = SceneManager._scene;
        scene.fadeOutAll();
        DataManager.loadGame(slotId);
        gameInterpreter.command115(); // 今のイベントが継続しないように中断コマンドを実行する
        Scene_Load.prototype.reloadMapIfUpdated.apply(scene);
        SceneManager.goto(Scene_Map);
        $gameSystem.onAfterLoad();
    };

    SaveCommand.runCommandSave = function (_, slotId) {
        if($gameSystem._saveInterval > 0){
            return;
        }
        
        $gameSystem._saveInterval = 15; // ロード時に再びセーブするプラグインコマンドから実行されるため、その回避
        Esrt.autoSave.isAutoSave    = true;
        
        $gameSystem.onBeforeSave();
        if (DataManager.saveGame(slotId)) {
            StorageManager.cleanBackup(slotId);
            this.onSaveSuccess();
        }else{
            this.onSaveFailure();
        }
        // $gameMap._events = typeof TN_gameMap ? TN_gameMap : $gameMap._events; // TN_LightSaveData_Map.jsとの競合回避
    };

    SaveCommand.runCommandRemove = function (_, slotId) {
        StorageManager.remove(slotId);
    };

    var upstream_Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function (command, args) {
        if (command === 'SaveCommand') {
            var type = (args[0] || '').trim();
            var slotId = SaveCommand.parseSlotId((args[1] || '').trim());
            SaveCommand.runCommand(this, type, slotId);
            return;
        }
        upstream_Game_Interpreter_pluginCommand.apply(this, arguments);
    };

    // -------------------------------------------------------------------------
    global.Torigoya = (global.Torigoya || {});
    global.Torigoya.SaveCommand = SaveCommand;
    
    
    //=============================================================================
    // ■white_esorat：独自追記
    //   SaveCommand
    //      セーブ成否ウィンドウの表示・メッセージの指定を追記します。
    //=============================================================================
    // オートセーブ成功メッセージの表示
    SaveCommand.onSaveSuccess = function() {
        if(Esrt.autoSave.isAutoSave){
            Esrt.autoSave.isShow        = true;
            Esrt.autoSave.isSaveSuccess = true;
        }
        Esrt.autoSave.isAutoSave    = false;
        
    };

    // オートセーブ失敗メッセージの表示
    SaveCommand.onSaveFailure = function() {
        if(Esrt.autoSave.isAutoSave){
            Esrt.autoSave.isShow        = true;
            Esrt.autoSave.isSaveSuccess = false;
        }
        Esrt.autoSave.isAutoSave    = false;
    };
    
    //=============================================================================
    // ■white_esorat：独自追記
    // セーブファイル1をオートセーブ専用として表示する
    //=============================================================================
    var _Scene_Save_onSavefileOk = Scene_Save.prototype.onSavefileOk;
    Scene_Save.prototype.onSavefileOk = function() {
        if(this.savefileId() == 1){
            this.onSaveFailure();
            return;
        }
        _Scene_Save_onSavefileOk.call(this);
    };
    
    // セーブ時にフラグを渡す。渡したフラグを元にオートセーブの文字列を灰色にする処理が行われる。
    var _Scene_Save_createListWindow = Scene_Save.prototype.createListWindow;
    Scene_Save.prototype.createListWindow = function() {
        _Scene_Save_createListWindow.call(this);
        this._listWindow.setSaveFlag();
        this._listWindow.refresh();
    };
    
    Window_SavefileList.prototype.drawItem = function(index) {
        var id = index + 1;
        
        
        var valid = DataManager.isThisGameFile(id);
        var info = DataManager.loadSavefileInfo(id);
        var rect = this.itemRectForText(index);
        this.resetTextColor();
        if (this._mode === 'load') {
            this.changePaintOpacity(valid);
        }
        this.drawFileId(id, rect.x, rect.y);
        if (info) {
            this.changePaintOpacity(valid);
            if(this._saveFlag && id == 1){
                this.changeTextColor(this.textColor(8));
            }
            this.drawContents(info, rect, valid);
            this.changePaintOpacity(true);
        }
        this.resetTextColor();
    };
    
    Window_SavefileList.prototype.drawFileId = function(id, x, y) {
        if(id != 1){
            this.drawText(TextManager.file + ' ' + id, x, y, 180);
        }else{
            if(this._saveFlag){
                this.changeTextColor(this.textColor(8));
            }
            this.drawText("オートセーブ", x, y, 180);
            this.resetTextColor();
        }
    };
    Window_SavefileList.prototype.setSaveFlag = function() {
        this._saveFlag = true;
    };
        
    // オートセーブ(ファイル1)へのセーブ時、アクセスファイル位置を元に戻す
    var _DataManager_saveGame = DataManager.saveGame;
    DataManager.saveGame = function(savefileId) {
        var tmpSavefileId = this._lastAccessedId;
        
        var ret = _DataManager_saveGame.apply(this, arguments);
        
        this._lastAccessedId = (this._lastAccessedId == 1) ? tmpSavefileId : this._lastAccessedId;
        
        return ret;
    };
    
    var _Scene_Load_onLoadSuccess = Scene_Load.prototype.onLoadSuccess;
    Scene_Load.prototype.onLoadSuccess = function() {
        var info = DataManager.loadSavefileInfo(this.savefileId());
        
        if (info && info.lastMagic && info.lastMagic instanceof Array ) {
            Esrt.spellOrder.orderResult  = info.lastMagic;
        }
        _Scene_Load_onLoadSuccess.apply(this, arguments);
    };
})(this);
